#include <Eigen/Core>
#include <Eigen/LU> 
#include <vector>
#include <iostream>
#include "writer.hpp"
#include <stdexcept>


///
/// Uses the newton method to solve 
///
///    vNext = v + (dt/2) [F(v) - F(vNext)]
///
/// for vNext given the initial guess initialGuess.
///
/// The results will be stored to vNext.
///
/// \param[out] vNext at the end of the call, this will have vNext
/// \param[in] initialGuess the initial guess to use for newton
/// \param[in] v the previously obtained v (ie. v_n)
/// \param[in] k1 the reaction factor k1 (see exercise)
/// \param[in] k2 the reaction factor k2 (see exercise)
/// \param[in] h the step size
/// \param[in] tolerance the limit on when to stop the newton iterations
/// \param[in] maxIterations maximum number of iterations to perform.
///
void newtonSolve(Eigen::Vector4d& vNext,
	const Eigen::Vector4d& initialGuess,
	const Eigen::Vector4d& v,
	double k1, double k2, double dt, double tolerance,
	int maxIterations) {

	vNext = initialGuess;
	
	// NOTE: This will not compile out of the box, you need to replace 
	// The comments with something meaningful.
	for (/* Setup the max number of iterations*/) {
		// Compute residual G(u)
		Eigen::Vector4d residual;
		
		// HINT: residual.norm() gives you the norm of the residual.
		if (/* TODO: Check if we can stop the Newton iteration*/ ) {
			return;
		}

		Eigen::Matrix4d DG; /*Compute DG here*/;
		// NOTE: You can use the following to set DG:
		//
		// DG << a11, a12, a13, a14,
		//       a21, a22, a23, a24,
		//       a31, a32, a33, a34,
		//       a41, a42, a43, a44;
		//
		// Where aij is the value of the i-th row and j-th column.
		
		// You can solve DG deltaX = residual with the following command
		// Vector4d deltaX = DG.lu().solve(residual);
		

		// Update vNext
		vNext = /* Formula for vNext here */
	}

	// If we reach this point, something wrong happened.
	throw std::runtime_error("Did not reach tolerance in Newton iteration");
}

int main() {
	
	double T = 20.0;
	size_t N = 1000;
	double dt = T / N;

	// This makes saving the results a lot easier. u1[n] will hold u1 at timestep n
	std::vector<double> u1(N+1), u2(N+1), u3(N+1), u4(N+1), time(N+1);

	// Set initial start values
	u1[0] = 0.2; 
	u2[0] = 0.3;
	u3[0] = 0.1;
	u4[0] = 0.4;
	time[0] = 0;

	// From the exercise
	double k1 = 0.4; 
	double k2 = 0.1;

	for (/**Correct time loop here*/) {
		Eigen::Vector4d vPrevious;
		vPrevious << u1[n], u2[n], u3[n], u4[n];

		Eigen::Vector4d vNext;
		// Compute vNext here

		
		// Store the results
		u1[n + 1] = vNext[0];
		u2[n + 1] = vNext[1];
		u3[n + 1] = vNext[2];
		u4[n + 1] = vNext[3];

		time[n + 1] = time[n] + dt;
	}
	writeToFile("u1.txt", u1);
	writeToFile("u2.txt", u2);
	writeToFile("u3.txt", u3);
	writeToFile("u4.txt", u4);
	writeToFile("time.txt", time);
	return 0;
}
